#ifdef HAVE_CONFIG_H
#  include <uim/config.h>
#endif
#include <gtk/gtk.h>

#include <uim/uim.h>
#include <uim/uim-helper.h>


static GtkWidget *hbox;
static GtkWidget *tmp_button = NULL;
static GList *menu_buttons = NULL;

static int read_tag;
static int uim_fd;
static GtkWidget *helper_parent_widget = NULL;

static gboolean
prop_button_pressed(GtkButton *prop_button, GdkEventButton *event, gpointer *user_data);
static gboolean
prop_button_released(GtkButton *prop_button, GdkEventButton *event, gpointer *user_data);
static void
menu_button_position(GtkMenu *menu, gint *x, gint *y, gboolean *push_in,
		     gpointer *user_data);

static void
activate_event(gpointer callback_data, guint ca, GtkWidget *widget);

static gchar *
translate(const gchar *path, gpointer data)
{
  return gettext(path);
}

static void
activate_event(gpointer callback_data, guint ca, GtkWidget *widget)
{
  GString *tmp = g_string_new((gchar*)callback_data);
  g_string_prepend(tmp, "prop_activate\n");
  g_string_append(tmp, "\n");
  uim_helper_send_message(uim_fd, tmp->str);
  g_string_free(tmp, TRUE);
}

static void
button_destroy(gpointer data, gpointer user_data)
{
  gtk_widget_destroy(GTK_WIDGET(data));
}


static void
helper_disconnect_cb(void)
{
  uim_fd = -1;
  gdk_input_remove(read_tag);
}


static void
helper_applet_prop_list_update(gchar **tmp)
{
  GtkItemFactory *mode_item_fact = NULL; /* quiet gcc */
  GtkWidget *button;
  int i = 0;
  gchar **tmp2 = NULL;
  gchar *charset = NULL;
  GString *path;

  tmp2 = g_strsplit(tmp[1], "=", 0);

  if(tmp2 && tmp2[0] && tmp2[1] && strcmp("charset", tmp2[0]) == 0) {
    charset = g_strdup(tmp2[1]);
    g_strfreev(tmp2);
  } else {
    g_strfreev(tmp2);
    return;
  }

  if(menu_buttons) {
    g_list_foreach(menu_buttons, button_destroy, NULL);
    g_list_free(menu_buttons);
    menu_buttons = NULL;
  }

  while(tmp[i] && strcmp("", tmp[i]) != 0) {
  
    tmp2 = g_strsplit(tmp[i], "\t", 0);

    if(tmp2 && tmp2[0]) 
      {
	if(strcmp("branch", tmp2[0]) == 0) {
	  
	  if(tmp_button) {
	    gtk_widget_destroy(tmp_button); tmp_button = NULL;
	  }

	  button = gtk_button_new_with_label(tmp2[1]);
	  gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
	  menu_buttons = g_list_append(menu_buttons, button);
	  
	  mode_item_fact = gtk_item_factory_new(GTK_TYPE_MENU, "<main>", NULL);
	  
	  g_signal_connect(G_OBJECT(button), "button-press-event",
			   G_CALLBACK(prop_button_pressed), mode_item_fact);

	  g_signal_connect(G_OBJECT(button), "button-release-event",
			   G_CALLBACK(prop_button_released), mode_item_fact);
	  
	  gtk_item_factory_set_translate_func(mode_item_fact, translate, NULL, NULL);
	  gtk_widget_show(button);
	} else if(strcmp("leaf", tmp2[0]) == 0) {
	  GtkItemFactoryEntry entry = {NULL, NULL, G_CALLBACK(activate_event), 0, ""};
	  path = g_string_new("");
	  g_string_printf(path, "/%s", tmp2[2]);
	  entry.path = path->str;
	  gtk_item_factory_create_item ( mode_item_fact, &entry, tmp2[4], 1);
	  g_string_free(path, TRUE);
	}
	g_strfreev(tmp2);
      }
    i++;
  }

  if(charset)
    g_free(charset);

}


static void
helper_applet_prop_label_update(gchar **tmp)
{
  GtkWidget *button;
  int i = 0;
  gchar **tmp2 = NULL;
  gchar *charset = NULL;

  if(tmp && tmp[1] ) {
    tmp2 = g_strsplit(tmp[1], "=", 0);
  } else {
    return;
  }

  if(tmp2 && tmp2[0] && tmp2[1] && strcmp("charset", tmp2[0]) == 0) {
    charset = g_strdup(tmp2[1]);
    g_strfreev(tmp2);
  } else {
    g_strfreev(tmp2);
    return;
  }

  while(tmp[i] && strcmp("", tmp[i]) != 0) {
    i++;
  }

  if(!menu_buttons || i - 2 != g_list_length(menu_buttons)) {
    uim_helper_client_get_prop_list();
    return;
  }
  
  i = 1;

  while(tmp[i] && strcmp("", tmp[i]) != 0) {
    tmp2 = g_strsplit(tmp[i], "\t", 0);
    
    if(tmp2 && tmp2[0] && tmp2[1])
      {
	button = g_list_nth_data(menu_buttons, i - 2 );
	gtk_button_set_label(button, tmp2[0]);
	g_strfreev(tmp2);
      }
    i++;
  }
}


static void
helper_applet_parse_helper_str(char *str)
{
  gchar **lines;
    
  lines = g_strsplit(str, "\n", 0);
    
  if(lines && lines[0]) {
    if( strcmp("prop_list_update", lines[0]) == 0) {
      helper_applet_prop_list_update(lines);
    } else if( strcmp("prop_label_update", lines[0]) == 0) {
      helper_applet_prop_label_update(lines);
    }
    g_strfreev(lines);
  }
}

static void
uim_applet_fd_read_cb(gpointer p, gint fd, GdkInputCondition c)
{
  char *tmp;
  
  uim_helper_read_proc(fd);
  while ((tmp = uim_helper_get_message())) {

    helper_applet_parse_helper_str(tmp);
    g_free(tmp); tmp = NULL;
  }

}

static gboolean
prop_button_pressed(GtkButton *prop_button, GdkEventButton *event,
		    gpointer *user_data)
{
 
  if(event->button == 2 || event->button == 3) {
  if(helper_parent_widget)
      gtk_propagate_event(GTK_WIDGET(helper_parent_widget), (GdkEvent *) event);
 }
  return FALSE;
}


static gboolean
prop_button_released(GtkButton *prop_button, GdkEventButton *event,
		    gpointer *user_data)
{
  int x,y;


  if(uim_fd < 0) {
    uim_fd = uim_helper_init_client_fd(helper_disconnect_cb);
    if(uim_fd > 0)
      read_tag = gdk_input_add(uim_fd, (GdkInputCondition)GDK_INPUT_READ,
			       uim_applet_fd_read_cb, 0);    
  }
  
  if(!event) {
    return FALSE;
  } else if(event->button == 2 || event->button == 3) {
    if(helper_parent_widget)
      gtk_propagate_event(GTK_WIDGET(helper_parent_widget), (GdkEvent *) event);
    return FALSE;
  }

  if(!user_data)
    return FALSE;

  menu_button_position(NULL, &x, &y, NULL, prop_button);
  gtk_item_factory_popup(user_data, x, y, NULL, NULL);
  return TRUE;
}

static void
menu_button_position(GtkMenu *menu, gint *x, gint *y, gboolean *push_in,
			  gpointer *user_data)
{
        GtkWidget *widget;
        gint wheight;
        gint wx, wy;
	gint sc_he , sc_wi; /*screen height, width*/

	g_return_if_fail(x && y);
 	g_return_if_fail(GTK_IS_BUTTON(user_data));

	widget = GTK_WIDGET(user_data);

	gdk_window_get_origin(widget->window, x, y);

        wheight = widget->requisition.height;

	sc_he = gdk_screen_get_height(gdk_screen_get_default ());
	sc_wi = gdk_screen_get_width (gdk_screen_get_default ());

	/*	if(*y + wy + wheight < sc_he){
	  *y = *y + wy + wheight;
	} else {
	  *y = *y - wy - wheight;
	}

	if(*x + wx < sc_wi){
	  *x = *x + wx;
	} else {
	  *x = *x - wx;
	  }*/
}

void
uim_helper_toolbar_set_parent_widget(GtkWidget *toolbar, GtkWidget *parent)
{
  helper_parent_widget = parent;
}

GtkWidget *
uim_helper_toolbar_new(void)
{
  hbox = gtk_hbox_new(FALSE, 0);

  uim_helper_client_get_prop_list();

  if(!menu_buttons){
    tmp_button = gtk_button_new_with_label("?");
    gtk_button_set_relief(GTK_BUTTON(tmp_button), GTK_RELIEF_HALF);
  
    g_signal_connect(G_OBJECT(tmp_button), "button_press_event",
		     G_CALLBACK(prop_button_pressed), NULL);
    
    gtk_box_pack_start(GTK_BOX(hbox), tmp_button, FALSE, FALSE, 0);
  
  }
  uim_fd     = uim_helper_init_client_fd(helper_disconnect_cb);

  if(uim_fd > 0) {
    read_tag = gdk_input_add(uim_fd, (GdkInputCondition)GDK_INPUT_READ,
			     uim_applet_fd_read_cb, 0);
    //    uim_helper_client_get_prop_list();
  }

  return hbox; 
}
